/* In en.DM00083560.pdf:   */

#define __IO volatile

/* p77 */
#define PERIPH_BASE	((uint32_t)0x40000000)
#define APB1PERIPH_BASE (PERIPH_BASE)
#define APB2PERIPH_BASE (PERIPH_BASE + 0x00010000)
#define AHB1PERIPH_BASE	(PERIPH_BASE + 0x00020000)
#define AHB2PERIPH_BASE	(PERIPH_BASE + 0x08000000)

/* 6.4 p223 */
/* Reset and Clock Control */
typedef struct {
__IO uint32_t CR;		/* 0x00 */
__IO uint32_t ICSCR;		/* 0x04 */
__IO uint32_t CFGR;		/* 0x08 */
__IO uint32_t PLLCFGR;		/* 0x0c */
__IO uint32_t PLLSAI1CFGR;	/* 0x10 */
__IO uint32_t PLLSAI2CFGR;	/* 0x14 */
__IO uint32_t CIER;		/* 0x18 */
__IO uint32_t CIFR;		/* 0x1c */
__IO uint32_t CICR;		/* 0x20 */
__IO uint32_t RESERVED0;	/* 0x24 */
__IO uint32_t AHB1RSTR;		/* 0x28 */
__IO uint32_t AHB2RSTR;		/* 0x2c */
__IO uint32_t AHB3RSTR;		/* 0x30 */
__IO uint32_t RESERVED1;	/* 0x34 */
__IO uint32_t APB1RSTR1;	/* 0x38 */
__IO uint32_t APB1RSTR2;	/* 0x3c */
__IO uint32_t APB2RSTR;		/* 0x40 */
__IO uint32_t RESERVED2;	/* 0x44 */
__IO uint32_t AHB1ENR;		/* 0x48 */
__IO uint32_t AHB2ENR;		/* 0x4c */
__IO uint32_t AHB3ENR;		/* 0x50 */
__IO uint32_t RESERVED3;	/* 0x54 */
__IO uint32_t APB1ENR1;		/* 0x58 */
__IO uint32_t APB1ENR2;		/* 0x5c */
__IO uint32_t APB2ENR;		/* 0x60 */
__IO uint32_t RESERVED4;	/* 0x64 */
__IO uint32_t AHB1SMENR;	/* 0x68 */
__IO uint32_t AHB2SMENR;	/* 0x6C */
__IO uint32_t AHB3SMENR;	/* 0x70 */
__IO uint32_t RESERVED5;	/* 0x74 */
__IO uint32_t APB1SMENR1;	/* 0x78 */
__IO uint32_t APB1SMENR2;	/* 0x7C */
__IO uint32_t APB2SMENR;	/* 0x80 */
__IO uint32_t RESERVED6;	/* 0x84 */
__IO uint32_t CCIPR;		/* 0x88 */
__IO uint32_t RESERVED7;	/* 0x8C */
__IO uint32_t BDCR;		/* 0x90 */
__IO uint32_t CSR;		/* 0x94 */

} RCC_TypeDef;

#define RCC_CR_MSION		(1<<0)
#define RCC_CR_MSIRGSEL		(1<<3)
#define RCC_CR_MSIRANGE		(0xf<<4)
#define RCC_CR_HSION		(1<<8)
#define RCC_CR_HSIRDY		(1<<10)
#define RCC_CR_HSERDY		(1<<17)
#define RCC_CR_HSEBYP		(1<<18)
#define RCC_CR_PLLON		(1<<24)
#define RCC_CR_PLLRDY		(1<<25)


#define RCC_CFGR_SW		(0x3)
#define RCC_CFGR_SW_PLL		(0x3)
#define RCC_CFGR_SW_HSI		(0x1)
#define RCC_CFGR_HPRE		(0xf<<4)

#define RCC_CFGR_SWS		(0x3<<2)
#define RCC_CFGR_SWS_PLL	(0x3<<2)

#define RCC_PLLCFGR_PLLSRC	(0x3)
#define RCC_PLLCFGR_PLLSRC_HSI	(0x2)
#define RCC_PLLCFGR_PLLN	(0xff<<8)
#define RCC_PLLCFGR_PLLM	(0x7<<4)
#define RCC_PLLCFGR_PLLR	(0x3<<25)
#define RCC_PLLCFGR_PLLREN	(1<<24)

#define RCC_AHB2ENR_GPIOAEN	(1<<0)
#define RCC_AHB2ENR_GPIOBEN	(1<<1)
#define RCC_AHB2ENR_GPIOCEN	(1<<2)
#define RCC_AHB2ENR_GPIODEN	(1<<3)
#define RCC_AHB2ENR_GPIOEEN	(1<<4)
#define RCC_AHB2ENR_GPIOFEN	(1<<5)
#define RCC_AHB2ENR_GPIOGEN	(1<<6)
#define RCC_AHB2ENR_GPIOHEN	(1<<7)
#define RCC_AHB2ENR_GPIOIEEN	(1<<8)

/* 6.4.19, p253 */
#define RCC_APB1ENR1_LCDEN	(1<<9)
#define RCC_APB1ENR1_PWREN	(1<<28)



/* 6.4.29, p272 */
#define RCC_BDCR_LSEON		(0x1<<0)	/* LSE Oscillator Enable */
#define RCC_BDCR_LSERDY		(0x1<<1)	/* LSE Oscillator Ready */
#define RCC_BDCR_LSEBYP		(0x1<<2)	/* LSE Oscillator Bypass */
#define RCC_BDCR_RTCSEL		(0x3<<8)
#define RCC_BDCR_RTCSEL_LSE	(0x1<<8)
#define RCC_BDCR_RTCSEL_LSI	(0x2<<8)
#define RCC_BDCR_RTCSEL_HSE	(0x3<<8)
#define RCC_BDCR_RTCEN		(0x1<<15)
#define RCC_BDCR_BDRST		(0x1<<16)	/* Backup domain sw reset */

#define	RCC_CSR_MSISRANGE	(0xf<<8)
#define RCC_CSR_LSION		(1<<0)


#define RCC_BASE	(AHB1PERIPH_BASE + 0x1000)
#define RCC		((RCC_TypeDef *)(RCC_BASE))

/* General Purpose IO */
typedef struct {
__IO uint32_t MODER;	/* 0x00 = Mode Register */
__IO uint32_t OTYPER;   /* 0x04 = Output Type Register */
__IO uint32_t OSPEEDR;	/* 0x08 = Output Speed Register */
__IO uint32_t PUPDR;	/* 0x0C = Pull Up/Pull Down Register */
__IO uint32_t IDR;	/* 0x10 = Input Data Register */
__IO uint32_t ODR;	/* 0x14 = Output Data Register */
__IO uint32_t BSRR;	/* 0x18 = Bit Set/Reset Register (LOW) */
__IO uint32_t LCKR;	/* 0x1C = Configuration Lock Register */
__IO uint32_t AFR[2];	/* 0x20 = Alternate Function (Low/High) Registers */
__IO uint32_t BRR;
__IO uint32_t ASCR;
} GPIO_TypeDef;

#define GPIOA_BASE	(AHB2PERIPH_BASE + 0x0000)
#define GPIOA ((GPIO_TypeDef *)GPIOA_BASE)

#define GPIOB_BASE	(AHB2PERIPH_BASE + 0x0400)
#define GPIOB ((GPIO_TypeDef *)GPIOB_BASE)

#define GPIOC_BASE	(AHB2PERIPH_BASE + 0x0800)
#define GPIOC ((GPIO_TypeDef *)GPIOC_BASE)

#define GPIOD_BASE	(AHB2PERIPH_BASE + 0x0C00)
#define GPIOD ((GPIO_TypeDef *)GPIOD_BASE)

#define GPIOE_BASE	(AHB2PERIPH_BASE + 0x1000)
#define GPIOE ((GPIO_TypeDef *)GPIOE_BASE)

#define MODER_INPUT	0x00
#define MODER_OUTPUT	0x01
#define MODER_ALT	0x02
#define MODER_ANALOG	0x03

#define MODER_SET(_s,_v)	(((_v&0x3) << (_s*2))

#define FLASH_BASE	((uint32_t)0x08000000U)

#define RESET	0


/* RTC -- Chapter 38.6 (p1238)*/
typedef struct {
__IO uint32_t TR;	/* 0x00 = Time Register */
__IO uint32_t DR;	/* 0x04 = Date Register */
__IO uint32_t CR;	/* 0x08 = Control Register */
__IO uint32_t ISR;	/* 0x0C = Init and Status Register */
__IO uint32_t PRER;	/* 0x10 = Prescalar Register */
__IO uint32_t WUTR;	/* 0x14 = Wakeup Time Register */
__IO uint32_t RESERVED0;/* 0x18 */
__IO uint32_t ALRMAR;	/* 0x1C = Alarm A register */
__IO uint32_t ALRMBR;	/* 0x20 = Alarm B register */
__IO uint32_t WPR;	/* 0x24 = Write Protect register */
} RTC_TypeDef;

#define RTC_BASE	(APB1PERIPH_BASE + 0x2800)
#define RTC ((RTC_TypeDef *)RTC_BASE)

/* LCD -- Chapter 25.6 (p767,p790) */
typedef struct {
__IO uint32_t CR;		/* 0x00 = Control Register */
__IO uint32_t FCR;		/* 0x04 = Frame Control Register */
__IO uint32_t SR;		/* 0x08 = Status Register */
__IO uint32_t CLR;		/* 0x0C = Clear Register */
__IO uint32_t RESERVED0;	/* 0x10 = Unused */
__IO uint32_t RAM[16];		/* 0x14 - 0x50 */
} LCD_TypeDef;

#define LCD_BASE	(APB1PERIPH_BASE + 0x2400)
#define LCD ((LCD_TypeDef *)LCD_BASE)

/* p790 */
#define LCD_CR_LCDEN		(0x1<<0)
#define LCD_CR_VSEL		(0x1<<1)
#define LCD_CR_DUTY_MASK	(0x7<<2)
#define LCD_CR_DUTY_1_4		(0x3<<2)
#define LCD_CR_BIAS_MASK	(0x3<<5)
#define LCD_CR_BIAS_1_3		(0x2<<5)
#define LCD_CR_MUX_SEG		(0x1<<7)


/* p791 */
#define LCD_FCR_PON_MASK	(0x7<<4)
#define LCD_FCR_PON7		(0x7<<4)
#define LCD_FCR_CC_MASK		(0x7<<10)
#define LCD_FCR_CC_LCD7		(0x7<<10)

/* p794 */
#define LCD_SR_ENS		(0x1<<0)
#define LCD_SR_UDR		(0x1<<2)
#define LCD_SR_UDD		(0x1<<3)
#define LCD_SR_RDY		(0x1<<4)
#define LCD_SR_FCRSF		(0x1<<5)




/* Power -- 5.4 p184 */
typedef struct {
__IO uint32_t CR1;	/* 0x00 = Control Register 1 */
__IO uint32_t CR2;	/* 0x04 = Control Register 2 */
__IO uint32_t CR3;	/* 0x08 = Control Register 3 */
__IO uint32_t CR4;	/* 0x0C = Control Register 4 */
} PWR_TypeDef;

/* Table 1 p80 */
#define PWR_BASE	(APB1PERIPH_BASE + 0x7000)
#define PWR ((PWR_TypeDef *)PWR_BASE)

#define PWR_CR1_DBP	(1<<8)	/* Disable Backup Write Protection */
